package org.rcisoft.bus.bcustomer.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.util.ExportToExcelUtil;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.bus.bcustomer.entity.BCustomer;
import org.rcisoft.bus.bcustomer.service.BCustomerService;

import java.util.List;

/**
* Created by cy on 2024年5月11日 下午3:19:37.
*/
@RestController
@RequestMapping("/bcustomer")
public class BCustomerController extends CyPaginationController<BCustomer> {


    @Autowired
    private BCustomerService bCustomerServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:stomer:add')")
    @CyOpeLogAnno(title = "system-BCustomer管理-新增BCustomer", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加BCustomer", notes="添加BCustomer")
    @PostMapping(value = "/add")
    public CyResult add(@Valid BCustomer bCustomer, BindingResult bindingResult) {
        CyPersistModel data = bCustomerServiceImpl.persist(bCustomer);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        bCustomer);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:stomer:delete')")
    @CyOpeLogAnno(title = "system-BCustomer管理-删除BCustomer", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="逻辑删除BCustomer", notes="逻辑删除BCustomer")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable Integer businessId,BCustomer bCustomer) {
        bCustomer.setBusinessId(businessId);
        CyPersistModel data = bCustomerServiceImpl.removeLogical(bCustomer);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:stomer:delete')")
    @CyOpeLogAnno(title = "system-BCustomer管理-删除BCustomer", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除BCustomer", notes="删除BCustomer")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable Integer businessId,BCustomer bCustomer) {
        bCustomer.setBusinessId(businessId);
        CyPersistModel data = bCustomerServiceImpl.remove(bCustomer);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:stomer:update')")
    @CyOpeLogAnno(title = "system-BCustomer管理-修改BCustomer", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改BCustomer", notes="修改BCustomer")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable Integer businessId, @Valid BCustomer bCustomer, BindingResult bindingResult) {
        bCustomer.setBusinessId(businessId);
        CyPersistModel data = bCustomerServiceImpl.merge(bCustomer);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        bCustomer);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:stomer:query')")
    @CyOpeLogAnno(title = "system-BCustomer管理-查询BCustomer", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一BCustomer", notes="查询单一BCustomer")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable String businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        bCustomerServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:stomer:list')")
    @CyOpeLogAnno(title = "system-BCustomer管理-查询BCustomer", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询BCustomer集合", notes="查询BCustomer集合")
    @GetMapping(value = "/queryBCustomers")
    public CyResult queryBCustomers(BCustomer bCustomer) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            bCustomerServiceImpl.findAll(bCustomer));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:stomer:list')")
    @CyOpeLogAnno(title = "system-BCustomer管理-查询BCustomer", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询BCustomer集合", notes="分页查询BCustomer集合")
    @GetMapping(value = "/queryBCustomerByPagination")
    public CyGridModel listByPagination(BCustomer bCustomer) {
        bCustomerServiceImpl.findAllByPagination(getPaginationUtility(), bCustomer);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-BCustomer管理-查询BCustomer", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出BCustomer信息", notes = "导出BCustomer信息")
    @GetMapping(value = "/export")
    public void outBCustomer(HttpServletResponse response,BCustomer bCustomer,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="BCustomer信息.xls";break;
            case "1": excelName="BCustomer信息.xlsx";break;
            case "2": excelName="BCustomer信息.csv";break;
        }
        List<BCustomer> bCustomerList = bCustomerServiceImpl.export(bCustomer);
        ExportToExcelUtil.exportExcel(bCustomerList, "BCustomer信息", "BCustomer信息", BCustomer.class, excelName, response);
    }
}
