package org.rcisoft.bus.bpnsypn.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.util.ExportToExcelUtil;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.bus.bpnsypn.entity.BPnSypn;
import org.rcisoft.bus.bpnsypn.service.BPnSypnService;

import java.util.List;

/**
* Created by cy on 2024年5月11日 下午3:19:37.
*/
@RestController
@RequestMapping("/bpnsypn")
public class BPnSypnController extends CyPaginationController<BPnSypn> {


    @Autowired
    private BPnSypnService bPnSypnServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:sypn:add')")
    @CyOpeLogAnno(title = "system-BPnSypn管理-新增BPnSypn", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加BPnSypn", notes="添加BPnSypn")
    @PostMapping(value = "/add")
    public CyResult add(@Valid BPnSypn bPnSypn, BindingResult bindingResult) {
        CyPersistModel data = bPnSypnServiceImpl.persist(bPnSypn);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        bPnSypn);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:sypn:delete')")
    @CyOpeLogAnno(title = "system-BPnSypn管理-删除BPnSypn", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="逻辑删除BPnSypn", notes="逻辑删除BPnSypn")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable Integer businessId,BPnSypn bPnSypn) {
        bPnSypn.setBusinessId(businessId);
        CyPersistModel data = bPnSypnServiceImpl.removeLogical(bPnSypn);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:sypn:delete')")
    @CyOpeLogAnno(title = "system-BPnSypn管理-删除BPnSypn", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除BPnSypn", notes="删除BPnSypn")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable Integer businessId,BPnSypn bPnSypn) {
        bPnSypn.setBusinessId(businessId);
        CyPersistModel data = bPnSypnServiceImpl.remove(bPnSypn);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:sypn:update')")
    @CyOpeLogAnno(title = "system-BPnSypn管理-修改BPnSypn", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改BPnSypn", notes="修改BPnSypn")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable Integer businessId, @Valid BPnSypn bPnSypn, BindingResult bindingResult) {
        bPnSypn.setBusinessId(businessId);
        CyPersistModel data = bPnSypnServiceImpl.merge(bPnSypn);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        bPnSypn);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:sypn:query')")
    @CyOpeLogAnno(title = "system-BPnSypn管理-查询BPnSypn", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一BPnSypn", notes="查询单一BPnSypn")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable Integer businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        bPnSypnServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:sypn:list')")
    @CyOpeLogAnno(title = "system-BPnSypn管理-查询BPnSypn", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询BPnSypn集合", notes="查询BPnSypn集合")
    @GetMapping(value = "/queryBPnSypns")
    public CyResult queryBPnSypns(BPnSypn bPnSypn) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            bPnSypnServiceImpl.findAll(bPnSypn));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:sypn:list')")
    @CyOpeLogAnno(title = "system-BPnSypn管理-查询BPnSypn", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询BPnSypn集合", notes="分页查询BPnSypn集合")
    @GetMapping(value = "/queryBPnSypnByPagination")
    public CyGridModel listByPagination(BPnSypn bPnSypn) {
        bPnSypnServiceImpl.findAllByPagination(getPaginationUtility(), bPnSypn);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-BPnSypn管理-查询BPnSypn", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出BPnSypn信息", notes = "导出BPnSypn信息")
    @GetMapping(value = "/export")
    public void outBPnSypn(HttpServletResponse response,BPnSypn bPnSypn,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="BPnSypn信息.xls";break;
            case "1": excelName="BPnSypn信息.xlsx";break;
            case "2": excelName="BPnSypn信息.csv";break;
        }
        List<BPnSypn> bPnSypnList = bPnSypnServiceImpl.export(bPnSypn);
        ExportToExcelUtil.exportExcel(bPnSypnList, "BPnSypn信息", "BPnSypn信息", BPnSypn.class, excelName, response);
    }
}
