package org.rcisoft.bus.bpnsypn.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.bus.bpnsypn.dao.BPnSypnRepository;
import org.rcisoft.bus.bpnsypn.entity.BPnSypn;
import org.rcisoft.bus.bpnsypn.service.BPnSypnService;
import org.rcisoft.core.service.CyBaseService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2024年5月11日 下午3:19:37.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BPnSypnServiceImpl extends ServiceImpl<BPnSypnRepository,BPnSypn>  implements BPnSypnService {


    /**
    * 保存
    * @param bPnSypn
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(BPnSypn bPnSypn){
        // pn验重
        if (baseMapper.querySomePn(bPnSypn) > 0) {
            throw new CyServiceException(501,"该pn已存在,请重新输入");
        }
        //增加操作
        int line = baseMapper.insert(bPnSypn);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            bPnSypn.getBusinessId()+"的信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除
    * @param bPnSypn
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(BPnSypn bPnSypn){
        int line = baseMapper.realDelete(bPnSypn);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            bPnSypn.getBusinessId()+"的信息");
        return new CyPersistModel(line);
    }
    /**
    * 逻辑删除
    * @param bPnSypn
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(BPnSypn bPnSypn){
        bPnSypn.setDeleted();
        int line = baseMapper.deleteById(bPnSypn);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                bPnSypn.getBusinessId()+"的信息");
            return new CyPersistModel(line);
        }
    /**
    * 修改
    * @param bPnSypn
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(BPnSypn bPnSypn){
        // pn验重
        if (baseMapper.querySomePn(bPnSypn) > 0) {
            throw new CyServiceException(501,"该pn已存在,请重新输入");
        }
        int line = baseMapper.updateById(bPnSypn);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ bPnSypn.getBusinessId()+"的信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询
    * @param id
    * @return
    */
    @Override
    public BPnSypn findById(Integer id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询
    * @param bPnSypn
    * @return
    */
    @Override
    public IPage<BPnSypn> findAllByPagination(CyPageInfo<BPnSypn> paginationUtility,
        BPnSypn bPnSypn){
        return baseMapper.queryBPnSypnsPaged(paginationUtility,bPnSypn);

    }
    /**
    * 查询list
    * @param bPnSypn
    * @return
    */
    @Override
    public List<BPnSypn> findAll(BPnSypn bPnSypn){
        return baseMapper.queryBPnSypns(bPnSypn);

    }

    /**
    * 导出
    * @return
    */
    @Override
    public List<BPnSypn> export(BPnSypn bPnSypn) {
        List<BPnSypn> bPnSypnList = baseMapper.queryBPnSypns(bPnSypn);
        return bPnSypnList;
    }

}
