package org.rcisoft.bus.jnsp7xtm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import cn.afterturn.easypoi.excel.entity.params.ExcelExportEntity;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.rcisoft.bus.jnsp7xtm.dao.Jnsp7xtmRepository;
import org.rcisoft.bus.jnsp7xtm.dao.Jnsp7xtmStatisticsRepository;
import org.rcisoft.bus.jnsp7xtm.dto.Jnsp7xtmExportTotalDto;
import org.rcisoft.bus.jnsp7xtm.dto.Jnsp7xtmTableHeaderDto;
import org.rcisoft.bus.jnsp7xtm.dto.Jnsp7xtmDto;
import org.rcisoft.bus.jnsp7xtm.dto.Jnsp7xtmPageRequestDto;
import org.rcisoft.bus.jnsp7xtm.entity.Jnsp7xtm;
import org.rcisoft.bus.jnsp7xtm.enums.ColTransitionEnum;
import org.rcisoft.bus.jnsp7xtm.service.Jnsp7xtmService;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.util.ExcelExportStatisticStyler;
import org.rcisoft.util.ExportToExcelUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

/**
 * @Author: gy
 * @Date: 2024/5/7 11:25
 * @Description:
 */
@Slf4j
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class Jnsp7xtmServiceImpl extends ServiceImpl<Jnsp7xtmRepository, Jnsp7xtm> implements Jnsp7xtmService {

    @Autowired
    private Jnsp7xtmRepository jnsp74tmRepository;

    @Autowired
    private Jnsp7xtmStatisticsRepository jnsp74tmStatisticsRepository;

    @Autowired
    private Jnsp7xtmStatisticServiceImpl jnsp74tmStatisticService;

    // 获取每次写入excel的长度
    @Value("${jnsp7xtm.export.pageSize}")
    private Integer size;

    // 静态锁
    private static Lock lock = new ReentrantLock();

    /**
     * 7.弹出框查询
     **/
    @Override
    public IPage<Jnsp7xtm> findAllByPagination(CyPageInfo<Jnsp7xtm> paginationUtility, Jnsp7xtmPageRequestDto jnsp74tmPageRequestDto) {
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getClickNqy())) {
            //设置查询值
            jnsp74tmPageRequestDto.setNqy(jnsp74tmPageRequestDto.getClickNqy());
        }
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getKhdm())) {
            jnsp74tmPageRequestDto.setVagueKhdm("");
        }
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getKhm())) {
            jnsp74tmPageRequestDto.setVaguekhm("");
        }
        jnsp74tmPageRequestDto.setType("1");
        // 获取总值
        // 获取关联表语句
        List<String> joinSqlList = new ArrayList<>();
        String joinSql = "";
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getOneDimensionality())) {
            String originalOneDime = "";
            if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                originalOneDime = jnsp74tmPageRequestDto.getOneDimensionality().replaceAll(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
            }
            if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                originalOneDime = jnsp74tmPageRequestDto.getOneDimensionality().replaceAll(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
            }
            jnsp74tmPageRequestDto.setOneDimensionality(originalOneDime);
        }
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getTwoDimensionality())) {
            String originalTwoDime = "";
            if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                originalTwoDime = jnsp74tmPageRequestDto.getTwoDimensionality().replaceAll(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
            }
            if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                originalTwoDime = jnsp74tmPageRequestDto.getTwoDimensionality().replaceAll(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
            }
            jnsp74tmPageRequestDto.setTwoDimensionality(originalTwoDime);
        }
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getThreeDimensionality())) {
            String originalthreeDime = "";
            if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                originalthreeDime = jnsp74tmPageRequestDto.getThreeDimensionality().replaceAll(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
            }
            if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                originalthreeDime = jnsp74tmPageRequestDto.getThreeDimensionality().replaceAll(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
            }

            jnsp74tmPageRequestDto.setThreeDimensionality(originalthreeDime);
        }
        if (!joinSqlList.isEmpty()) {
            joinSql = joinSqlList.stream().distinct().collect(Collectors.joining(" "));
        }
        jnsp74tmPageRequestDto.setJoinSql(joinSql);
        IPage<Jnsp7xtm> jnsp74tmIPage = jnsp74tmRepository.queryJnsp74tmByPaged(paginationUtility, jnsp74tmPageRequestDto);
        return jnsp74tmIPage;
    }
    /**
     * 8.导出弹出框信息
     * **/
    @Override
    public void exportUserDetailJnsp74tm(Jnsp7xtmPageRequestDto jnsp74tmPageRequestDto, String excelName, HttpServletResponse response) {
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getClickNqy())) {
            //设置查询值
            jnsp74tmPageRequestDto.setNqy(jnsp74tmPageRequestDto.getClickNqy());
        }
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getKhdm())) {
            jnsp74tmPageRequestDto.setVagueKhdm("");
        }
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getKhm())) {
            jnsp74tmPageRequestDto.setVaguekhm("");
        }
        jnsp74tmPageRequestDto.setType("1");
        // 获取总值
        List<String> joinSqlList = new ArrayList<>();
        String joinSql = "";
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getOneDimensionality())) {
            String originalOneDime = "";
            if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                originalOneDime = jnsp74tmPageRequestDto.getOneDimensionality().replaceAll(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
            }
            if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                originalOneDime = jnsp74tmPageRequestDto.getOneDimensionality().replaceAll(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
            }
            jnsp74tmPageRequestDto.setOneDimensionality(originalOneDime);
        }
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getTwoDimensionality())) {
            String originalTwoDime = "";
            if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                originalTwoDime = jnsp74tmPageRequestDto.getTwoDimensionality().replaceAll(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
            }
            if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                originalTwoDime = jnsp74tmPageRequestDto.getTwoDimensionality().replaceAll(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
            }
            jnsp74tmPageRequestDto.setTwoDimensionality(originalTwoDime);
        }
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getThreeDimensionality())) {
            String originalthreeDime = "";
            if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                originalthreeDime = jnsp74tmPageRequestDto.getThreeDimensionality().replaceAll(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
            }
            if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                originalthreeDime = jnsp74tmPageRequestDto.getThreeDimensionality().replaceAll(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
            }
            jnsp74tmPageRequestDto.setThreeDimensionality(originalthreeDime);
        }
        if (!joinSqlList.isEmpty()) {
            joinSql = joinSqlList.stream().distinct().collect(Collectors.joining(" "));
        }
        jnsp74tmPageRequestDto.setJoinSql(joinSql);
        Integer total = jnsp74tmRepository.queryAllJnsp74tm(jnsp74tmPageRequestDto);
        if (total > 0) {
            int pageNumber = 1;
            if (total >= size) {
                pageNumber = (int) Math.ceil((double) total / size);
            }
            // 获取exportParams
            ExportParams exportParams = new ExportParams();
            exportParams.setStyle(ExcelExportStatisticStyler.class);
            String[] format = excelName.split("\\.");
            if (format[1].equals("xlsx")) {
                exportParams.setType(ExcelType.XSSF);
            }
            Workbook workbook = jnsp7xtmBigExcel(pageNumber, exportParams, null, jnsp74tmPageRequestDto);
            ExcelExportUtil.closeExportBigExcel();
            ExportToExcelUtil.downLoadExcel(excelName, response, workbook);
        } else {
            List<Jnsp7xtm> records = new ArrayList<>();
            ExportToExcelUtil.exportExcel(records, "详细信息", "详细信息", Jnsp7xtm.class, excelName, response);
        }

    }
    /**
     * 9.导出明细
     * **/
    @Override
    public void exportDetailJnsp74tm(Jnsp7xtmPageRequestDto jnsp74tmPageRequestDto, String excelName, HttpServletResponse response) {
        try {
            if (lock.tryLock(1000, TimeUnit.MILLISECONDS)) {
                try {
                    if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getKhdm())) {
                        jnsp74tmPageRequestDto.setVagueKhdm("");
                    }
                    if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getKhm())) {
                        jnsp74tmPageRequestDto.setVaguekhm("");
                    }
                    jnsp74tmPageRequestDto.setType("2");
                    // 获取总值
                    List<String> joinSqlList = new ArrayList<>();
                    String joinSql = "";
                    if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getOneDimensionality())) {
                        String originalOneDime = "";
                        if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                            originalOneDime = jnsp74tmPageRequestDto.getOneDimensionality().replaceAll(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
                        }
                        if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                            originalOneDime = jnsp74tmPageRequestDto.getOneDimensionality().replaceAll(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                            joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
                        }
                        jnsp74tmPageRequestDto.setOneDimensionality(originalOneDime);
                    }
                    if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getTwoDimensionality())) {
                        String originalTwoDime = "";
                        if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                            originalTwoDime = jnsp74tmPageRequestDto.getTwoDimensionality().replaceAll(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
                        }
                        if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                            originalTwoDime = jnsp74tmPageRequestDto.getTwoDimensionality().replaceAll(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                            joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
                        }
                        jnsp74tmPageRequestDto.setTwoDimensionality(originalTwoDime);
                    }
                    if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getThreeDimensionality())) {
                        String originalthreeDime = "";
                        if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                            originalthreeDime = jnsp74tmPageRequestDto.getThreeDimensionality().replaceAll(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
                        }
                        if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                            originalthreeDime = jnsp74tmPageRequestDto.getThreeDimensionality().replaceAll(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                            joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
                        }
                        jnsp74tmPageRequestDto.setThreeDimensionality(originalthreeDime);
                    }
                    if (!joinSqlList.isEmpty()) {
                        joinSql = joinSqlList.stream().distinct().collect(Collectors.joining(" "));
                    }
                    jnsp74tmPageRequestDto.setJoinSql(joinSql);
                    Integer total = jnsp74tmRepository.queryAllJnsp74tm(jnsp74tmPageRequestDto);
                    if (total > 0) {
                        int pageNumber = 1;
                        if (total >= size) {
                            pageNumber = (int) Math.ceil((double) total / size);
                        }
                        // 获取exportParams
                        ExportParams exportParams = new ExportParams();
                        exportParams.setStyle(ExcelExportStatisticStyler.class);
                        String[] format = excelName.split("\\.");
                        if (format[1].equals("xlsx")) {
                            exportParams.setType(ExcelType.XSSF);
                        }
                        Workbook workbook = jnsp7xtmBigExcel(pageNumber, exportParams, null, jnsp74tmPageRequestDto);
                        ExcelExportUtil.closeExportBigExcel();
                        ExportToExcelUtil.downLoadExcel(excelName, response, workbook);
                    } else {
                        List<Jnsp7xtm> records = new ArrayList<>();
                        ExportToExcelUtil.exportExcel(records, "详细信息", "详细信息", Jnsp7xtm.class, excelName, response);
                    }
                }  catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    lock.unlock();
                }
            } else {
                throw new CyServiceException(501,"该功能使用频繁,请稍后重试");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * 10.导出统计
     * **/
    @Override
    public void exportTotal(HttpServletResponse response, Jnsp7xtmPageRequestDto requestDto) {
        // 1.查询条件
        boolean conditionIsNull =
                requestDto.isRmb() ||
                        requestDto.isXsjg() ||
                        requestDto.isCount() || requestDto.isChzs() || requestDto.isChzsa() || requestDto.isChzsb() || requestDto.isZzc() || requestDto.isZzcg();
        if (!conditionIsNull) {
            requestDto.setChzs(true);
        }
        //查询月份
        List<String> monthsList =jnsp74tmStatisticsRepository.getMonths();
        String months = monthsList.stream()
                // 将每个月份拼接成形如 "[yyyyMM]" 的字符串
                .map(month -> "[" + month + "]")
                // 使用逗号连接每个月份字符串
                .collect(Collectors.joining(","));
        // 2.获取动态导出的表头和数据
        Jnsp7xtmExportTotalDto exportTotalDto = outTotalJnsp7xtm(requestDto,months,monthsList);
       if (!exportTotalDto.getDataList().isEmpty()) {
           ExportToExcelUtil.exportExcel(response, exportTotalDto.getEntityList(), exportTotalDto.getDataList());
       } else {
           throw new CyServiceException("要导出数据为空");
       }
    }

    private Workbook jnsp7xtmBigExcel(Integer pageNumber, ExportParams exportParams, Workbook workbook, Jnsp7xtmPageRequestDto jnsp74tmPageRequestDto) {
        for (int i = 1; i <= pageNumber; i++) {
            IPage<Jnsp7xtm> page = new Page<>(i,size);
            IPage<Jnsp7xtm> iPage = jnsp74tmRepository.queryFindAllList(page, jnsp74tmPageRequestDto);
            List<Jnsp7xtm> records = iPage.getRecords();
            workbook = ExcelExportUtil.exportBigExcel(exportParams, Jnsp7xtm.class, records);
        }
        return workbook;
    }
    /**
     * 获取导出表头和数据
     * **/
    public Jnsp7xtmExportTotalDto outTotalJnsp7xtm(Jnsp7xtmPageRequestDto jnsp74tmPageRequestDto, String months, List<String> monthsList) {
        Jnsp7xtmExportTotalDto exportTotalDto = new Jnsp7xtmExportTotalDto();
        List<Jnsp7xtmTableHeaderDto> tableHeaders = new ArrayList<>();
        List<Jnsp7xtmTableHeaderDto> sortTableHeaders = new ArrayList<>();
        List<Jnsp7xtmTableHeaderDto> chzsTableHeaders = new ArrayList<>();
        List<Jnsp7xtmTableHeaderDto> rmbTableHeaders = new ArrayList<>();
        List<Jnsp7xtmTableHeaderDto> chzsaTableHeaders = new ArrayList<>();
        List<Jnsp7xtmTableHeaderDto> chzsbTableHeaders = new ArrayList<>();
        /**
         * 处理表头
         * **/
        List<Jnsp7xtmTableHeaderDto> tableHeadeList = getTableHeadeList(jnsp74tmPageRequestDto);
        tableHeadeList.stream().forEach(item -> {
            tableHeaders.add(item);
            rmbTableHeaders.add(item);
            chzsaTableHeaders.add(item);
            chzsbTableHeaders.add(item);
            chzsTableHeaders.add(item);
        });
        Jnsp7xtmDto emptyJnsp74xmDto = jnsp74tmStatisticService.getEmptyJnsp74xmDto(jnsp74tmPageRequestDto);
        // 获取关联表语句
        List<String> joinSqlList = new ArrayList<>();
        String joinSql = "";
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getOneDimensionality())) {
            if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
            }
        }

        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getTwoDimensionalityValue())) {
            if (jnsp74tmPageRequestDto.getTwoDimensionalityValue().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
            }
        }

        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getThreeDimensionalityValue())) {
            if (jnsp74tmPageRequestDto.getThreeDimensionalityValue().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                joinSqlList.add("left join b_customer cust on cust.cust_code = j7xtm.khdm and cust.del_flag = '0'");
            }
        }
        if (!joinSqlList.isEmpty()) {
            joinSql = joinSqlList.stream().distinct().collect(Collectors.joining(" "));
        }
        jnsp74tmPageRequestDto.setJoinSql(joinSql);
        List<Jnsp7xtmDto> tempjnsp74TmDtoList = jnsp74tmStatisticsRepository.queryTotal(jnsp74tmPageRequestDto);
        if (!tempjnsp74TmDtoList.isEmpty()) {
            // 过滤出没有对应合计值年份
            List<String> filteredList = monthsList.stream()
                    .filter(month -> !tempjnsp74TmDtoList.stream().map(Jnsp7xtmDto::getNqy).collect(Collectors.toList()).contains(month))
                    .collect(Collectors.toList());
            if (!filteredList.isEmpty()) {
                for (int i = 0; i < filteredList.size(); i++) {
                    Jnsp7xtmDto one = new Jnsp7xtmDto();
                    BeanUtils.copyProperties(emptyJnsp74xmDto, one);
                    one.setNqy(filteredList.get(i));
                    tempjnsp74TmDtoList.add(one);
                }
            }
        }
        List<Jnsp7xtmDto> jnsp74tmDtos = tempjnsp74TmDtoList.stream().sorted(Comparator.comparing(Jnsp7xtmDto::getNqy)).collect(Collectors.toList());
        if (!jnsp74tmDtos.isEmpty()) {
            jnsp74tmDtos.forEach(item -> {
                if (item.getChzs() != null) {
                    Jnsp7xtmTableHeaderDto tableHeader = getTableHeader("chzs", item);
                    chzsTableHeaders.add(tableHeader);
                    sortTableHeaders.add(tableHeader);
                }
                if (item.getRmb() != null) {
                    Jnsp7xtmTableHeaderDto tableHeader = getTableHeader("rmb", item);
                    rmbTableHeaders.add(tableHeader);
                    sortTableHeaders.add(tableHeader);
                }
                if (item.getChzsa() != null) {
                    Jnsp7xtmTableHeaderDto tableHeader = getTableHeader("chzsa", item);
                    chzsaTableHeaders.add(tableHeader);
                    sortTableHeaders.add(tableHeader);
                }
                if (item.getChzsb() != null) {
                    Jnsp7xtmTableHeaderDto tableHeader = getTableHeader("chzsb", item);
                    chzsbTableHeaders.add(tableHeader);
                    sortTableHeaders.add(tableHeader);
                }
            });
        }
        List<Jnsp7xtmTableHeaderDto> tempList = sortTableHeaders.stream().sorted(Comparator.comparing(Jnsp7xtmTableHeaderDto::getOldKey)).collect(Collectors.toList());
        tempList.stream().forEach(item -> {
            tableHeaders.add(item);
        });
        List<ExcelExportEntity> entityList = new ArrayList<>();
        for (int i = 0; i < tableHeaders.size(); i++) {
            entityList.add(new ExcelExportEntity(tableHeaders.get(i).getName(), tableHeaders.get(i).getKey(), 20));
        }
        // 将对应的列传为数值
        for (int i = 0; i < entityList.size(); i++) {
            if (i > tableHeadeList.size() && i < entityList.size()) {
                entityList.get(i).setType(10);
            }
        }
        // 获取数据
        List<Map<String, Object>> dataList = new ArrayList<>();
        // 获取统计列
        String  statistics = "";
        // 获取排序列
        String orderColumn = "";
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getOneDimensionality())) {
            String statisticsOneOneDimensionality = "";
            String orderOneOneDimensionality = "";
            if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getOneDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
                orderOneOneDimensionality = jnsp74tmPageRequestDto.getOneDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(), "");
            }

            if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getOneDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                orderOneOneDimensionality = jnsp74tmPageRequestDto.getOneDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), "");
            }
            statistics = statisticsOneOneDimensionality;
            orderColumn = orderOneOneDimensionality;
        }

        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getTwoDimensionality())) {
            String statisticsOneOneDimensionality = "";
            String orderOneOneDimensionality = "";
            if (!"".equals(statistics)) {
                if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                    statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
                    orderOneOneDimensionality = jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(), "");
                }

                if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                    statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                    orderOneOneDimensionality = jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), "");
                }
                statistics = statistics +","+ statisticsOneOneDimensionality;
                orderColumn = orderColumn + "," + orderOneOneDimensionality;
            } else {
                if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                    statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(), ColTransitionEnum.JNSP7XTM.getValue());
                    orderOneOneDimensionality = jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(), "");
                }
                if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                    statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                    orderOneOneDimensionality = jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), "");
                }

                statistics = statisticsOneOneDimensionality;
                orderColumn = orderOneOneDimensionality;
            }
        }

        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getThreeDimensionality())) {
            String statisticsOneOneDimensionality = "";
            String orderOneOneDimensionality = "";
            if (!"".equals(statistics)) {
                if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                    statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(),ColTransitionEnum.JNSP7XTM.getValue());
                    orderOneOneDimensionality = jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(),"");
                }

                if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                    statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                    orderOneOneDimensionality = jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), "");
                }
                statistics = statistics + ","+ statisticsOneOneDimensionality;
                orderColumn = orderColumn + "," +orderOneOneDimensionality;
            } else {
                if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                    statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(),ColTransitionEnum.JNSP7XTM.getValue());
                    orderOneOneDimensionality = jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(),"");
                }

                if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                    statisticsOneOneDimensionality = jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), ColTransitionEnum.CUSTOMER.getValue());
                    orderOneOneDimensionality = jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(), "");
                }
                statistics = statisticsOneOneDimensionality;
                orderColumn = orderOneOneDimensionality;
            }
        }

        List<Map<String, Object>> exportListCount =  jnsp74tmStatisticsRepository.queryList(jnsp74tmPageRequestDto,months,1,10,"2","xsjg",statistics,orderColumn,joinSql);
        if (!exportListCount.isEmpty()) {
            for (int i=0; i<exportListCount.size(); i++) {
                HashMap<String, Object> contentMap = new HashMap<>();
                if (jnsp74tmPageRequestDto.isChzs()) {
                   getMap("j7xtm.chzsa+j7xtm.chzsb", statistics, i, chzsTableHeaders, months, jnsp74tmPageRequestDto,contentMap,orderColumn,joinSql);
                }

                if (jnsp74tmPageRequestDto.isRmb()) {
                     getMap("j7xtm.rmb", statistics, i, rmbTableHeaders, months, jnsp74tmPageRequestDto,contentMap,orderColumn,joinSql);

                }

                if (jnsp74tmPageRequestDto.isChzsa()) {
                    getMap("j7xtm.chzsa", statistics, i, chzsaTableHeaders, months, jnsp74tmPageRequestDto,contentMap,orderColumn,joinSql);
                }

                if (jnsp74tmPageRequestDto.isChzsb()) {
                    getMap("j7xtm.chzsb", statistics, i, chzsbTableHeaders, months, jnsp74tmPageRequestDto,contentMap,orderColumn,joinSql);
                }

                dataList.add(contentMap);
            }
        }
        exportTotalDto.setEntityList(entityList);
        exportTotalDto.setDataList(dataList);
        return exportTotalDto;
    }


    /**
     * 获取动态列的表头
     * **/
    private Jnsp7xtmTableHeaderDto getTableHeader(String tar, Jnsp7xtmDto item) {
        Jnsp7xtmTableHeaderDto tableHeader = new Jnsp7xtmTableHeaderDto();
        String key = item.getNqy() + "_" + tar;
        tableHeader.setName(key);
        tableHeader.setKey(key);
        tableHeader.setOldKey(item.getNqy());
        return tableHeader;
    }
    /**
     * 获取处理好的数据
     * **/
    private void getMap(String dynamicColumn, String statistics, int i, List<Jnsp7xtmTableHeaderDto> oneTable, String months, Jnsp7xtmPageRequestDto jnsp74tmPageRequestDto, HashMap<String, Object> contentMap,String orderColumn, String joinSql) {
        List<Map<String, Object>> maps = jnsp74tmStatisticsRepository.queryList(jnsp74tmPageRequestDto, months, 1, 10, "2", dynamicColumn, statistics,orderColumn,joinSql);
        for (int j = 0; j < oneTable.size(); j++) {
            for (Map.Entry<String, Object> objectEntry :maps.get(i).entrySet()) {
                if (objectEntry.getKey().equals(oneTable.get(j).getOldKey())) {
                    contentMap.put(oneTable.get(j).getKey(),objectEntry.getValue());
                }
            }
        }
    }

    /**
     * 获取固定列数据
     * **/
    private List<Jnsp7xtmTableHeaderDto> getTableHeadeList(Jnsp7xtmPageRequestDto jnsp74tmPageRequestDto) {
        /* 初始化 固定列的label和value
        fixedTable key 是数据库中的数据列  value是前端的显示列
        */
        HashMap<String,String> fixedTable = new HashMap<>();
        fixedTable.put("khdm","用户id");
        fixedTable.put("khm","用户名称");
        fixedTable.put("jypn","jypn");
        fixedTable.put("jlpm","jlpm");
        fixedTable.put("xslj","xslj");
        fixedTable.put("cust_io","custIo");
        fixedTable.put("cust_g0","custG0");
        List<Jnsp7xtmTableHeaderDto> list = new ArrayList<>();
        // 维度一
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getOneDimensionality())) {
            String originalOneDime = "";
            if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                originalOneDime =  jnsp74tmPageRequestDto.getOneDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(),"");
            }
            if (jnsp74tmPageRequestDto.getOneDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                originalOneDime =  jnsp74tmPageRequestDto.getOneDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(),"");
            }
            Jnsp7xtmTableHeaderDto oneTableHeader = new Jnsp7xtmTableHeaderDto();
            oneTableHeader.setName(fixedTable.get(originalOneDime));
            oneTableHeader.setKey(originalOneDime);
            oneTableHeader.setOldKey(originalOneDime);
            list.add(oneTableHeader);
        }
        // 维度二
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getTwoDimensionality())) {
            String originalTwoDime = "";
            if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                originalTwoDime =  jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(),"");
            }
            if (jnsp74tmPageRequestDto.getTwoDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                originalTwoDime =  jnsp74tmPageRequestDto.getTwoDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(),"");
            }
            Jnsp7xtmTableHeaderDto oneTableHeader = new Jnsp7xtmTableHeaderDto();
            oneTableHeader.setName(fixedTable.get(originalTwoDime));
            oneTableHeader.setKey(originalTwoDime);
            oneTableHeader.setOldKey(originalTwoDime);
            list.add(oneTableHeader);
        }
        // 维度三
        if (StringUtils.isNotBlank(jnsp74tmPageRequestDto.getThreeDimensionality())) {
            String originalThreeDime = "";
            if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.JNSP7XTM.getCode())) {
                originalThreeDime =  jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.JNSP7XTM.getCode(),"");
            }
            if (jnsp74tmPageRequestDto.getThreeDimensionality().contains(ColTransitionEnum.CUSTOMER.getCode())) {
                originalThreeDime =  jnsp74tmPageRequestDto.getThreeDimensionality().replace(ColTransitionEnum.CUSTOMER.getCode(),"");
            }
            Jnsp7xtmTableHeaderDto oneTableHeader = new Jnsp7xtmTableHeaderDto();
            oneTableHeader.setName(fixedTable.get(originalThreeDime));
            oneTableHeader.setKey(originalThreeDime);
            oneTableHeader.setOldKey(originalThreeDime);
            list.add(oneTableHeader);
        }
        return list;
    }
}
