package org.rcisoft.bus.jnsp7xtmdqsj.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.Stopwatch;
import com.mongodb.client.model.Collation;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.session.SqlSession;
import org.apache.poi.ss.usermodel.Workbook;
import org.rcisoft.bus.jnsp7xtm.dto.Jnsp7xtmDto;
import org.rcisoft.bus.jnsp7xtm.dto.Jnsp7xtmOptionDto;
import org.rcisoft.bus.jnsp7xtm.entity.Jnsp7xtm;
import org.rcisoft.bus.jnsp7xtm.enums.ColTransitionEnum;
import org.rcisoft.bus.jnsp7xtm.vo.Jnsp7xtmVo;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.bus.jnsp7xtmdqsj.dao.Jnsp7xtmDqsjRepository;
import org.rcisoft.bus.jnsp7xtmdqsj.entity.Jnsp7xtmDqsj;
import org.rcisoft.bus.jnsp7xtmdqsj.service.Jnsp7xtmDqsjService;
import org.rcisoft.core.service.CyBaseService;

import org.rcisoft.util.ExcelExportStatisticStyler;
import org.rcisoft.util.ExportToExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2024年5月14日 上午10:20:48.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class Jnsp7xtmDqsjServiceImpl extends ServiceImpl<Jnsp7xtmDqsjRepository,Jnsp7xtmDqsj>  implements Jnsp7xtmDqsjService {

    // 数据表重置锁
    private static Lock dataTableResetLock = new ReentrantLock();

    // 月度表拉取锁
    private static Lock monthlyTablePullLock = new ReentrantLock();

    @Value("${jnsp7xtmDqsj.batchAddPagesize}")
    private Integer batchInsertSize;

    @Value("${jnsp7xtmDqsj.shema}")
    private String shema;

    /**
    * 保存
    * @param jnsp7xtmDqsj
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(Jnsp7xtmDqsj jnsp7xtmDqsj){
        //增加操作
        int line = baseMapper.insert(jnsp7xtmDqsj);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            jnsp7xtmDqsj.getBusinessId()+"的信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除
    * @param jnsp7xtmDqsj
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(Jnsp7xtmDqsj jnsp7xtmDqsj){
        int line = baseMapper.realDelete(jnsp7xtmDqsj);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            jnsp7xtmDqsj.getBusinessId()+"的信息");
        return new CyPersistModel(line);
    }

    /**
    * 修改
    * @param jnsp7xtmDqsj
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(Jnsp7xtmDqsj jnsp7xtmDqsj){
        int line = baseMapper.updateById(jnsp7xtmDqsj);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ jnsp7xtmDqsj.getBusinessId()+"的信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询
    * @param id
    * @return
    */
    @Override
    public Jnsp7xtmDqsj findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询
    * @param jnsp7xtmDqsj
    * @return
    */
    @Override
    public IPage<Jnsp7xtmDqsj> findAllByPagination(CyPageInfo<Jnsp7xtmDqsj> paginationUtility,
        Jnsp7xtmDqsj jnsp7xtmDqsj){
        return baseMapper.queryJnsp7xtmDqsjsPaged(paginationUtility,jnsp7xtmDqsj);

    }

    /**
    * 查询list
    * @param jnsp7xtmDqsj
    * @return
    */
    @Override
    public List<Jnsp7xtmDqsj> findAll(Jnsp7xtmDqsj jnsp7xtmDqsj){
        return baseMapper.queryJnsp7xtmDqsjs(jnsp7xtmDqsj);

    }

    /**
    * 导出
    * @return
    */
    @Override
    public List<Jnsp7xtmDqsj> export(Jnsp7xtmDqsj jnsp7xtmDqsj) {
        List<Jnsp7xtmDqsj> jnsp7xtmDqsjList = baseMapper.queryJnsp7xtmDqsjs(jnsp7xtmDqsj);
        return jnsp7xtmDqsjList;
    }

    @Override
    public Jnsp7xtmOptionDto getMonth() {
        Jnsp7xtmOptionDto jnsp7xtmOptionDto = new Jnsp7xtmOptionDto();
        List<String> monthList = baseMapper.getMonthList();
        jnsp7xtmOptionDto.setLabel("month");
        jnsp7xtmOptionDto.setPullDownList(monthList);
        return jnsp7xtmOptionDto;
    }

    // 数据表重置
    @Override
    public CyPersistModel dataTableRese() {
        try {
            if (dataTableResetLock.tryLock(1000, TimeUnit.MILLISECONDS)) {
                try {
                    baseMapper.truncateJnsp7xtmDqsj(shema);
                }  catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    dataTableResetLock.unlock();
                }
            } else {
                throw new CyServiceException(501,"该功能使用频繁,请稍后重试");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return new CyPersistModel(1);
    }

    /**
     * 月度表重置
     * **/
    @Override
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    public CyPersistModel monthlyTablePull() {
        try {
            if (monthlyTablePullLock.tryLock(1000, TimeUnit.MILLISECONDS)) {
                try {
                    // 创建并启动计时器
                    Stopwatch stopwatch = Stopwatch.createStarted();
                    // 获取全部数据
                    List<Jnsp7xtmDqsj> addList = new ArrayList<>();
                    // khdm is not null && pn is not null
                    baseMapper.streamQuery(resultContext -> {
                        Jnsp7xtmVo resultObject = resultContext.getResultObject();
                        Jnsp7xtmDqsj jnsp7xtmDqsj = Jnsp7xtmDqsj.generateJnsp7xtmDqsj(resultObject);
                        addList.add(jnsp7xtmDqsj);
                        if (addList.size() > batchInsertSize) {
                            baseMapper.batchAddJnsp7xtmDqsj(addList);
                            addList.clear();
                        }
                    });
                    if(CollectionUtil.isNotEmpty(addList))
                        baseMapper.batchAddJnsp7xtmDqsj(addList);
                    // 停止计时器
                    stopwatch.stop();
                    log.info("查询耗时秒" + stopwatch.elapsed().getSeconds());
                    log.info("查询耗时毫秒" + stopwatch.elapsed(TimeUnit.MILLISECONDS));
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new CyServiceException(501,"数据同步异常");
                }
                finally {
                    monthlyTablePullLock.unlock();
                }
            } else {
                throw new CyServiceException(501,"该功能使用频繁,请稍后重试");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return new CyPersistModel(1);
    }

}
